************************************************************************
*
*     Random_nPDFs
*
*     Generation of Monte Carlo replicas for EPS09
*     
*     The user must choose: 
*                          A       : nucleus               ; integer 
*                          order   : 1=LO, 2=NLO           ; integer
*                          nrep    : number of MC replicas ; integer
*
*     Compile together with EPS09.f
*
************************************************************************

      program Random_nPDF
      implicit none

      integer A, order, pset, nrep, neig, k, irep, ieig, jran
      integer ipdf,npdf,keig, ix, nx, nq, nxx, nnx, iq
      
ccccc PARAMETERS THAT THE USER MUST CHOOSE
      parameter(A=208) ! Lead nucleus
      parameter(nrep=1000) ! Number of generated replicas
      parameter(order=2) ! NLO      

ccccc Dimension of the x and Q^2 grid
      parameter(nx=50,nQ=50)

ccccc DO NOT CHANGE
      parameter(neig=15) ! Number of eigenvector pairs
      parameter(npdf=8) ! Number of partonic species


      double precision  x, Q, ruv, rdv, ru, rd, rs, rc, rb, rg
      double precision sum_rg, Q2(0:nq)
      double precision random(neig,npdf,nrep), gasdev, r
      double precision xmin, xmax
      double precision hessian_error(npdf), r_central(npdf)
      double precision diff_central(npdf), diff_err(npdf)
      double precision err(npdf)
      double precision ruv_rep_a(0:nx,0:nQ,nrep)
      double precision rdv_rep_a(0:nx,0:nQ,nrep)
      double precision ru_rep_a(0:nx,0:nQ,nrep)
      double precision rd_rep_a(0:nx,0:nQ,nrep)
      double precision rs_rep_a(0:nx,0:nQ,nrep)
      double precision rc_rep_a(0:nx,0:nQ,nrep)
      double precision rg_rep_a(0:nx,0:nQ,nrep)
      double precision rb_rep_a(0:nx,0:nQ,nrep)
      double precision ruv_rep_s(0:nx,0:nQ,nrep)
      double precision rdv_rep_s(0:nx,0:nQ,nrep)
      double precision ru_rep_s(0:nx,0:nQ,nrep)
      double precision rd_rep_s(0:nx,0:nQ,nrep)
      double precision rs_rep_s(0:nx,0:nQ,nrep)
      double precision rc_rep_s(0:nx,0:nQ,nrep)
      double precision rg_rep_s(0:nx,0:nQ,nrep)
      double precision rb_rep_s(0:nx,0:nQ,nrep) 
      double precision ruv_eig(0:nx,0:nq,2*neig)
      double precision rdv_eig(0:nx,0:nq,2*neig)
      double precision ru_eig(0:nx,0:nq,2*neig)
      double precision rd_eig(0:nx,0:nq,2*neig)
      double precision rs_eig(0:nx,0:nq,2*neig)
      double precision rc_eig(0:nx,0:nq,2*neig)
      double precision rg_eig(0:nx,0:nq,2*neig)
      double precision rb_eig(0:nx,0:nq,2*neig)
      double precision ruv_eigen(0:nx,0:nq,2*neig)
      double precision rdv_eigen(0:nx,0:nq,2*neig)
      double precision ru_eigen(0:nx,0:nq,2*neig)
      double precision rd_eigen(0:nx,0:nq,2*neig)
      double precision rs_eigen(0:nx,0:nq,2*neig)
      double precision rc_eigen(0:nx,0:nq,2*neig)
      double precision rg_eigen(0:nx,0:nq,2*neig) 
      double precision rb_eigen(0:nx,0:nq,2*neig)
      double precision ruv_central(0:nx,0:nQ), rdv_central(0:nx,0:nQ)
      double precision ru_central(0:nx,0:nQ), rc_central(0:nx,0:nQ)
      double precision rd_central(0:nx,0:nQ), rs_central(0:nx,0:nQ)
      double precision rg_central(0:nx,0:nQ), rb_central(0:nx,0:nQ)
      double precision Qmin,Qmax,apu,lstep
     
      real SECNDS

      Character (Len=50) filenimis,filenimia

*     ----------------------------------------------------------
*     ----------------------------------------------------------


      OPEN (11,file='gridq2_eps09.dat',status='OLD')
      Do k = 0,50
      Read(11,*) Q2(k)
      End Do
      Close(11)
c

      Qmin = 1.3d0
      Qmax = 1d3

!     Random number seed
      jran=-int(SECNDS(0.0))

!     Gaussian random numbers
!     SHould be fully correlated in x
!     Only differ between replicas an eigenvectors
      do ieig=1,neig
         do ipdf=1,npdf
            do irep=1,nrep
               random(ieig,ipdf,irep)=gasdev(jran)
            enddo
         enddo
      enddo

      do ix=0,nx

*     xgrid
         if(ix.le.25) then
!     log spacing for points below 0.1
            xmin=1d-6
            xmax = 1d-1
            nxx=25
            x=xmin*(xmax/xmin)**(dfloat(ix)/(nxx))
         else
            xmin=1d-1
            xmax=1d0
            x=xmin + (xmax - xmin)*(dfloat(ix-25)/(nxx))
         endif


            

         do iq=0,nQ

            Q=dsqrt(q2(iq))

            
!     Central value
            pset=1
            call EPS09(order, pset, A, x, Q,
     &           ruv, rdv, ru, rd, rs, rc, rb, rg)
            ruv_central(ix,iq) = ruv
            rdv_central(ix,iq) = rdv
            ru_central(ix,iq) = ru
            rd_central(ix,iq) = rd
            rs_central(ix,iq) = rs
            rc_central(ix,iq) = rc
            rb_central(ix,iq) = rb
            rg_central(ix,iq) = rg
         
*     Initialization
            do irep=1,nrep
               rg_rep_a(ix,iQ,irep)=rg_central(ix,iq)
               ruv_rep_a(ix,iQ,irep)=ruv_central(ix,iq)
               rdv_rep_a(ix,iQ,irep)=rdv_central(ix,iq)
               ru_rep_a(ix,iQ,irep)=ru_central(ix,iq)
               rd_rep_a(ix,iQ,irep)=rd_central(ix,iq)
               rs_rep_a(ix,iQ,irep)=rs_central(ix,iq)
               rc_rep_a(ix,iQ,irep)=rc_central(ix,iq)
               rb_rep_a(ix,iQ,irep)=rb_central(ix,iq)
               
               rg_rep_s(ix,iQ,irep)=rg_central(ix,iq)
               ruv_rep_s(ix,iQ,irep)=ruv_central(ix,iq)
               rdv_rep_s(ix,iQ,irep)=rdv_central(ix,iq)
               ru_rep_s(ix,iQ,irep)=ru_central(ix,iq)
               rd_rep_s(ix,iQ,irep)=rd_central(ix,iq)
               rs_rep_s(ix,iQ,irep)=rs_central(ix,iq)
               rc_rep_s(ix,iQ,irep)=rc_central(ix,iq)
               rb_rep_s(ix,iQ,irep)=rb_central(ix,iq)              
            enddo

           
!     Generate replicas
               
               do pset=2,neig*2+1
                  
                  call EPS09(order, pset, A, x, Q,
     &                 ruv, rdv, ru, rd, rs, rc, rb, rg)
               
                  rg_eig(ix,iq,pset-1)=rg
                  ruv_eig(ix,iq,pset-1)=ruv
                  rdv_eig(ix,iq,pset-1)=rdv
                  ru_eig(ix,iq,pset-1)=ru
                  rd_eig(ix,iq,pset-1)=rd
                  rs_eig(ix,iq,pset-1)=rs
                  rc_eig(ix,iq,pset-1)=rc
                  rb_eig(ix,iq,pset-1)=rb
                  
               enddo
           enddo
           enddo               
               
            do ix=0,nx
            do iq=0,nq     
            do irep=1,nrep
            do ieig=1,neig

!     Correlation between PDFs
                  r =random(ieig,1,irep) 
                  if(r.ge.0d0) then
                     keig=2*ieig-1
                  else
                     keig=2*ieig
                  endif

                          
                  rg_eigen(ix,iq,keig) = rg_eig(ix,iq,keig)
                  ruv_eigen(ix,iq,keig) = ruv_eig(ix,iq,keig)
                  rdv_eigen(ix,iq,keig) = rdv_eig(ix,iq,keig)
                  ru_eigen(ix,iq,keig) = ru_eig(ix,iq,keig)
                  rd_eigen(ix,iq,keig) = rd_eig(ix,iq,keig)
                  rs_eigen(ix,iq,keig) = rs_eig(ix,iq,keig)
                  rc_eigen(ix,iq,keig) = rc_eig(ix,iq,keig)
                  rb_eigen(ix,iq,keig) = rb_eig(ix,iq,keig)

ccccccc      ASYMMETRIC
                  
           rg_rep_a(ix,iQ,irep)=rg_rep_a(ix,iQ,irep)
     1     + (rg_eigen(ix,iq,keig)-rg_central(ix,iq)) * dabs(r)
           ruv_rep_a(ix,iQ,irep)=ruv_rep_a(ix,iQ,irep) 
     1     + (ruv_eigen(ix,iq,keig)-ruv_central(ix,iq)) * dabs(r)
           rdv_rep_a(ix,iQ,irep)=rdv_rep_a(ix,iQ,irep) 
     1     + (rdv_eigen(ix,iq,keig)-rdv_central(ix,iq)) * dabs(r)
           ru_rep_a(ix,iQ,irep)=ru_rep_a(ix,iQ,irep) 
     1     + (ru_eigen(ix,iq,keig)-ru_central(ix,iq)) * dabs(r)
           rd_rep_a(ix,iQ,irep)=rd_rep_a(ix,iQ,irep) 
     1     + (rd_eigen(ix,iq,keig)-rd_central(ix,iq)) * dabs(r)
           rs_rep_a(ix,iQ,irep)=rs_rep_a(ix,iQ,irep) 
     1     + (rs_eigen(ix,iq,keig)-rs_central(ix,iq)) * dabs(r)
           rc_rep_a(ix,iQ,irep)=rc_rep_a(ix,iQ,irep) 
     1     + (rc_eigen(ix,iq,keig)-rc_central(ix,iq)) * dabs(r)
           rb_rep_a(ix,iQ,irep)=rb_rep_a(ix,iQ,irep) 
     1     + (rb_eigen(ix,iq,keig)-rb_central(ix,iq)) * dabs(r)
     
ccccccc      SYMMETRIC
     
                  rg_rep_s(ix,iQ,irep)=rg_rep_s(ix,iQ,irep) 
     1        + (rg_eig(ix,iq,2*ieig)-rg_eig(ix,iq,2*ieig-1))*r*0.5
                  ruv_rep_s(ix,iQ,irep)=ruv_rep_s(ix,iQ,irep) 
     1        + (ruv_eig(ix,iq,2*ieig)-ruv_eig(ix,iq,2*ieig-1))*r*0.5d0
                  rdv_rep_s(ix,iQ,irep)=rdv_rep_s(ix,iQ,irep) 
     1        + (rdv_eig(ix,iq,2*ieig)-rdv_eig(ix,iq,2*ieig-1))*r*0.5d0
                  ru_rep_s(ix,iQ,irep)=ru_rep_s(ix,iQ,irep) 
     1        + (ru_eig(ix,iq,2*ieig)-ru_eig(ix,iq,2*ieig-1))*r*0.5d0
                  rd_rep_s(ix,iQ,irep)=rd_rep_s(ix,iQ,irep) 
     1        + (rd_eig(ix,iq,2*ieig)-rd_eig(ix,iq,2*ieig-1))*r*0.5d0
                  rs_rep_s(ix,iQ,irep)=rs_rep_s(ix,iQ,irep) 
     1        + (rs_eig(ix,iq,2*ieig)-rs_eig(ix,iq,2*ieig-1))*r*0.5d0
                  rc_rep_s(ix,iQ,irep)=rc_rep_s(ix,iQ,irep) 
     1        + (rc_eig(ix,iq,2*ieig)-rc_eig(ix,iq,2*ieig-1))*r*0.5d0
                  rb_rep_s(ix,iQ,irep)=rb_rep_s(ix,iQ,irep) 
     1        + (rb_eig(ix,iq,2*ieig)-rb_eig(ix,iq,2*ieig-1))*r*0.5d0
     
                  
               enddo            ! End sum over eigenvectors
               
               
            enddo               ! End loop over replica generation
         
               
         enddo

      enddo
      

 441  format(f15.10,1x,1050(f9.3,1x))


*****************************************
*     Store the random PDFs in two files, 
*     one for the symmetric and 
*     one for the asymmetric
*     prescription
*****************************************

      If (order .EQ. 1) then
      
     
         If (A < 10) Then
           Write(filenimis,'("EPS09LOR_", I1,"_MC_s")'), A
           Write(filenimia,'("EPS09LOR_", I1,"_MC_a")'), A

         Else If (A < 100) Then
           Write(filenimis,'("EPS09LOR_", I2,"_MC_s")'), A
           Write(filenimia,'("EPS09LOR_", I2,"_MC_a")'), A

         Else If (A < 1000) Then
           Write(filenimis,'("EPS09LOR_", I3,"_MC_s")'), A
           Write(filenimia,'("EPS09LOR_", I3,"_MC_a")'), A
        
         End If

      ElseIf (order .EQ. 2) then

        If (A < 10) Then
           Write(filenimis,'("EPS09NLOR_", I1,"_MC_s")'), A
           Write(filenimia,'("EPS09NLOR_", I1,"_MC_a")'), A
           
        Else If (A < 100) Then
           Write(filenimis,'("EPS09NLOR_", I2,"_MC_s")'), A
           Write(filenimia,'("EPS09NLOR_", I2,"_MC_a")'), A
           
        Else If (A < 1000) Then
           Write(filenimis,'("EPS09NLOR_", I3,"_MC_s")'), A
           Write(filenimia,'("EPS09NLOR_", I3,"_MC_a")'), A
           
        End If

      End If
      
      open(unit=20,status="unknown",file=filenimis)
      open(unit=30,status="unknown",file=filenimia)
      
      do irep=1,nrep
         do iQ=0,nQ
            Q=dsqrt(q2(iq))

            write(20,445) Q**2d0
            write(30,445) Q**2d0
            
            do ix=0,nx
               write(20,445) ruv_rep_s(ix,iQ,irep),
     1              rdv_rep_s(ix,iQ,irep),
     1              ru_rep_s(ix,iQ,irep),
     1              rd_rep_s(ix,iQ,irep),
     1              rs_rep_s(ix,iQ,irep),
     1              rc_rep_s(ix,iQ,irep),
     1              rb_rep_s(ix,iQ,irep),
     1              rg_rep_s(ix,iQ,irep)
     
               write(30,445) ruv_rep_a(ix,iQ,irep),
     1              rdv_rep_a(ix,iQ,irep),
     1              ru_rep_a(ix,iQ,irep),
     1              rd_rep_a(ix,iQ,irep),
     1              rs_rep_a(ix,iQ,irep),
     1              rc_rep_a(ix,iQ,irep),
     1              rb_rep_a(ix,iQ,irep),
     1              rg_rep_a(ix,iQ,irep)
     
            enddo
         enddo
      enddo
      close(20)
      close(30)
      
 445  format(8(1x,1PE11.3))

      end

************************************************
************************************************

***********************************************
*
*     ran.f
*
*     Random number generators
*
**********************************************


      function GASDEV(idum)
c     Gaussian random number generator
      INTEGER idum,iset
      DOUBLE PRECISION gasdev
      DOUBLE PRECISION fac,gset,rsq,w1,w2,ran1
      SAVE iset,gset
      DATA iset/0/
      if (iset.eq.0) then
1       w1=2.*ran1(idum)-1.
        w2=2.*ran1(idum)-1.
        rsq=w1**2+w2**2
        if(rsq.ge.1..or.rsq.eq.0.)goto 1
        fac=sqrt(-2.*log(rsq)/rsq)
        gset=w1*fac
        gasdev=w2*fac
        iset=1
      else
        gasdev=gset
        iset=0
      endif
      return
      END

cccccccccccccccccccccccccccccccccccccccccccc

      function RAN1(idum)
      INTEGER idum,IA,IM,IQ,IR,NTAB,NDIV
      DOUBLE PRECISION ran1,AM,EPS,RNMX
      PARAMETER (IA=16807,IM=2147483647,AM=1./IM,IQ=127773,IR=2836,
     *NTAB=32,NDIV=1+(IM-1)/NTAB,EPS=1.2e-7,RNMX=1.-EPS)
      INTEGER j,k,iv(NTAB),iy
      SAVE iv,iy
      DATA iv /NTAB*0/, iy /0/
      if (idum.le.0.or.iy.eq.0) then
        idum=max(-idum,1)
        do 11 j=NTAB+8,1,-1
          k=idum/IQ
          idum=IA*(idum-k*IQ)-IR*k
          if (idum.lt.0) idum=idum+IM
          if (j.le.NTAB) iv(j)=idum
 11      continue
        iy=iv(1)
      endif
      k=idum/IQ
      idum=IA*(idum-k*IQ)-IR*k
      if (idum.lt.0) idum=idum+IM
      j=1+iy/NDIV
      iy=iv(j)
      iv(j)=idum
      ran1=min(AM*iy,RNMX)
      return
      END

cccccccccccccccccccccccccccccccccccccc
c
c     Dirty random number generators
c
cccccccccccccccccccccccccccccccccccccc

      function DIRTY(jran)
      implicit double precision(a-h,o-y)
      jran=mod(jran*106+1283,6075)
      dirty=dfloat(jran)/dfloat(6075)
      end
      
      function IDIRTY(jran,jlo,jhi)
      implicit double precision(a-h,o-y)
      jran=mod(jran*106+1283,6075)
      idirty=jlo+((jhi-jlo+1)*jran)/6075
      end

      function IDIRTY1(jran3,jlo,jhi)
      implicit double precision(a-h,o-y)
      jran3=mod(jran3*1366+150889,714025)
      idirty1=jlo+((jhi-jlo+1)*jran3)/714025
      end

cccccccccccccccccccccccccccccccccccccccccccccccc


      function ran2(idum)
      implicit none
*     RAN2 random number generator from Numerical Recipees
      integer idum,im1,im2,imm1,ia1,ia2,iq1,iq2,ir1,ir2,ntab,ndiv
      double precision ran2,am,eps,rnmx
      parameter(im1=2147483563,im2=2147483399)
      parameter(am=1d0/im1,imm1=im1-1d0,ia1=40014)
      parameter(ia2=40692,iq1=53668,iq2=52774,ir1=12211,ir2=3791)
      parameter(ntab=32,ndiv=1+imm1/ntab,eps=1.2d-7,rnmx=1d0-eps)
*     Long perios (> 2e18) random number generator of L'Ecuyer
*     with Bath-Durham shuffle and addedd safeguards. Returns a uniform
*     random number deviate between 0d0 and 1d0 (exclusive of the
*     endpoint values). Call with idum a NEGATIVE INTEGER to initialize;
*     thereafter, do not alter idum between successive deviates
*     in a sequence. RNMX should approximate the largest
*     floating value which is less that 1
      integer idum2,j,k,iv(ntab),iy
      save iv,iy,idum2
      data idum2/123456789/, iv/ntab*0/,iy/0/
      if(idum.le.0) then ! Initialize
         idum=max(-idum,1) ! Be sure to prevent idum=0
         idum2=idum
         do 11 j=ntab+8,1,-1 ! Load shuffke table
            k=idum/iq1
            idum=ia1*(idum-k*iq1)-k*ir1
            if(idum.lt.0) idum=idum+im1
            if(j.le.ntab) iv(j)=idum
 11      enddo
         iy=iv(1)
      endif
      k=idum/iq1 ! Start here when not initialzing
      idum=ia1*(idum-k*iq1)-k*ir1
      if(idum.lt.0) idum=idum+im1
      k=idum2/iq2
      idum2=ia2*(idum2-k*iq2)-k*ir2
      if(idum2.lt.0) idum2=idum2+im2
      j=1+iy/ndiv
      iy=iv(j)-idum2
      iv(j)=idum
      if(iy.lt.1) iy=iy+imm1
      ran2=min(am*iy,rnmx)   ! Users do not expect endpoint values
      
      return
      end

********************************************

